/*
This code contains 2 separate functions - SE8_demo() and board functional test code.
If either SW1 or SW2 is presesd as the device exits RESET, the Functional Test routine
executes.  If neither push-button SW is pressed during RESET, the SE8_demo code 
executes.  

*/

#include <hidef.h>			 // for EnableInterrupts macro */
#include "derivative.h"		 // include peripheral declarations */
#include "sci.h"				 // Serial COM declarations

#define SW1 	PTAD_PTAD2
#define SW2 	PTAD_PTAD3

#define LED1 	PTAD_PTAD0
#define LED2	PTAD_PTAD1

#define LED00	PTAD = 0x00
#define LED01	PTAD = 0x01
#define LED10	PTAD = 0x02
#define LED11	PTAD = 0x03

#define  CLKS_FLL_PLL 0x00
#define  INT_REF  0x04


// PROTOTYPES
void init(void);				// init device
void init_KBI(void);			// init KBI
void init_ADC(void);			// init ADC
void ICS_FEI_8M(void);		// setup clock 
void SE8_demo(void);			// SE8 Demo Code



void main(void) {
	
	SOPT1 = 0x23;				// STOPE,BKGDPE, RSTPE set

	PTAPE = PTAPE_PTAPE3_MASK | PTAPE_PTAPE2_MASK;	// enable PTA[3:2] pull ups
	while(!(PTAD & 0x0C));	// wait until PTA pull ups are enabled
	
	if(SW1 & SW2) SE8_demo();	// execute demo code unless SW1 AND SW2 pressed
	
	EnableInterrupts;			// enable interrupts  
	ICS_FEI_8M();				// internal clock @ 8MHz, FLL engaged
	init();						// init device
	init_KBI();					// init KBI

	for(;;) {

		// Serial Loopback through the COM port
		if(SCIS1_RDRF){
			putchar(ReadSerialPort());	
		}
	}  
}

// initialize the target device
void init(void){
	
//	SOPT1 = 0x23;				// STOPE,BKGDPE, RSTPE set
	
	PTAD = 0x3F;				// init PTA 
	PTBD = 0x02;				// init PTB 
	PTCD = 0x00;				// init PTC

	PTADD = 0x33;				// PTA[7:6] input, PTA[5:0] output
	PTBDD = 0xFF;				// PTB[7:0] output
	PTCDD = 0xFF;				// PTC[7:0] output
	
}

// KBIisr is executed when either SW1 or SW2 is pressed
void interrupt 20 KBIisr(void){
	
	byte dummy;
	
	KBISC_KBIE = 0;			// disable KBI interrupts
	KBISC_KBACK = 1;			// ack interrupt
	dummy = PTAD;				// capture status of KBI inputs
	while(!SW1 | !SW2);		// wait for PBSW release
	if(!(dummy & 0x04)) init_ADC();	// toggle LED1
	if(!(dummy & 0x08)){
		ADCSC1 = 0x1F;			// disable ACD
		LED01;					// LED2 on
		ensci();					// enable COM port
	}
	KBISC_KBIE = 1;			// enable KBI interrupts
}

//  ADCisr() is executed when the ACD is enabled and a conversion is complete
void interrupt 23 ADCisr(void){
	
	byte dummy;
	
	dummy = ADCRL;				// get ADC conversion results
	
	if(dummy > 0xBD){
		LED00;					// both LEDs ON
		return;
	}
	if(dummy > 0x7E){
		LED01;					// LED2 ON
		return;
	}
	if(dummy > 0x3F){
		LED10;					// LED1 ON
		return;
	}

	LED11;						// both LEDs OFF
}

// enable keyboard interrupt on PTA[3:2] input, active low detect
void init_KBI(void){
	
	KBIES = 0;					// detect falling edge
	KBIPE = KBIPE_KBIPE3_MASK | KBIPE_KBIPE2_MASK;		// enable KBI[3:2] on pin
	KBISC_KBACK = 1;			// ack pending interrupt
	KBISC_KBIE = 1;			// enable KBI interrupt
}

// initialize ADC, input clock div 16, continuous conversion, interrupt enabled
void init_ADC(void){
	SCIC2 &= ~SCIC2_TE_MASK | ~SCIC2_RE_MASK;	// SCI Tx/Rx disabled
	ADCCFG = 0;
	ADCCFG = ADCCFG_ADIV_MASK | ADCCFG_ADICLK0_MASK;	// input clk div 8, bus clk div 2 is input 
	APCTL1 = APCTL1_ADPC4_MASK;		// enable ADP4 input
	ADCSC1 = 0x64;
//	ADCSC1 = ADCSC1_AIEN_MASK | ADCSC1_ADCO | ADCSC1_ADCH4_MASK;	// enable conversion complete interrupt, enable ADP4
}

//Setup clock, FLL Engaged Internal, with trim
void ICS_FEI_8M(void){
  if (NVICSTRM != 0xFF) {
    ICSTRM = NVICSTRM;		// capture calculated trim value
  }
  ICSC1 = CLKS_FLL_PLL | INT_REF;
  while (!ICSSC_IREFST);		// wait for Reference Status bit to update 
  while (ICSSC_CLKST != 0x0);	// wait for clock status bits to update
  ICSC2_BDIV  = 0;	
}

void SE8_demo(void){
	int cnt1, cnt2;
	SOPT1 = 0x23;

	DisableInterrupts;					// disable interrupts  

	PTAD = 0x3F;							// init PTA 
	PTBD = 0x02;							// init PTB 
	PTCD = 0x00;							// init PTC

   PTADD = 0x3F;							// PTA[7:6] input, PTA[5:0] output
   PTBDD = 0xFF;							// PTB[7:0] output
   PTCDD = 0xFF;							// PTC[7:0] output

	for(;;) {
		PTAD_PTAD0 ^= 1;						// LED 1 ON
		for(cnt1=0; cnt1<0x08; ++cnt1){
				PTAD_PTAD0 ^= 1;					// toggle LED 1
				PTAD_PTAD1 ^= 1;					// toggle LED2
			for(cnt2=0; cnt2<0xffff; ++cnt2){
			}
		}
	}  

}